/*BINFMTCXX: -Igeometry/include -Iboost_1_76_0
 */
#include <iostream>
#include <boost/geometry.hpp>
#include <boost/geometry/index/rtree.hpp>
#include <sys/time.h> // todo: use boost or std::chrono
#include <boost/endian/conversion.hpp>
#include <boost/interprocess/managed_mapped_file.hpp>
#include <boost/interprocess/mapped_region.hpp>

namespace bg = boost::geometry;
namespace bgm = bg::model;
namespace bgi = bg::index;
namespace ipc = boost::interprocess;
using namespace std;

// used types

typedef bgm::point<double, 2, bg::cs::cartesian> P;
typedef bgm::linestring<P> LS;
typedef bgm::box<P> B;
typedef pair<B, unsigned> value_t;
typedef bgi::linear<51, 25> params_t;
typedef bgi::indexable<value_t> indexable_t;
typedef bgi::equal_to<value_t> equal_to_t;
typedef ipc::allocator<value_t, ipc::managed_mapped_file::segment_manager> allocator_t;
typedef bgi::rtree<value_t,
                   params_t,
                   indexable_t,
                   equal_to_t,
                   allocator_t> rtree_t;

typedef ipc::managed_mapped_file mmfile;

uint64_t
timestamp()
{
  timeval s;
  gettimeofday(&s,NULL);
  return uint64_t(s.tv_sec)*1000000+uint64_t(s.tv_usec);
}

int
main(int argc, char* argv[]) {
  P p(1001875,6190443);

  mmfile f(ipc::open_read_only, argv[1]);
  std::pair<const rtree_t *, std::size_t> r = f.find<const rtree_t>("rtree");
  const rtree_t *t=r.first;
  cerr << "t=" << t << endl;
  cerr << "t->size()=" << t->size() << endl;

  for (unsigned i=0; i<3; ++i) {
    {
      uint64_t start=timestamp();
      rtree_t::const_query_iterator it = t->qbegin(bgi::nearest(p, 100));
      cerr << (timestamp()-start) << "usec" << endl;
      assert(it!=t->qend());
      cerr << bg::wkt(it->first) << " " << it->second << endl;
    }
    {
      uint64_t start=timestamp();
      rtree_t::const_query_iterator it = t->qbegin(bgi::nearest(p, t->size()));
      cerr << (timestamp()-start) << "usec" << endl;
      assert(it!=t->qend());
      cerr << bg::wkt(it->first) << " " << it->second << endl;
    }
  }
}
